/************************************************************************/
/*									*/
/*	ALL.PRO						21.7.93		*/
/*									*/
/*	Andreas Rieke, Heusnerstr. 24, 42283 Wuppertal.			*/
/*									*/
/*	Zusammenstellung aller passiven Protokolle:			*/
/*	  DIF_HEL\CHNG_PAS.PRO						*/
/*	  DSA\VERIFY.PRO						*/
/*	  EL_GAM\MULTIPLE\RECEIVE.PRO					*/
/*	  EL_GAM\SINGLE\RECEIVE.PRO					*/
/*	  FIA_SHA\AUTH_PAS.PRO						*/
/*	  FIA_SHA\USER_PAS.PRO						*/
/*	  MAS_OMU\MULTIPLE\RECEIVE.PRO					*/
/*	  MAS_OMU\SINGLE\RECEIVE.PRO					*/
/*	  PLAIN\RECEIVE.PRO						*/
/*	  RSA\MULTIPLE\RECEIVE.PRO					*/
/*	  RSA\SINGLE\RECEIVE.PRO					*/
/*									*/
/*	Vorgaben:							*/
/*	  user:      Name des Anwenders.				*/
/*	  bits_user: Laenge der Schluessel, mit denen der Partner 	*/
/*		     authentifiziert wird.				*/
/*	  anz:       Anzahl der Authentifikationsschritte, mit denen	*/
/*		     der Partner authentifiziert wird.			*/
/*	  public:    Pfad fuer oeffentliche Schluessel.			*/
/*	  private:   Pfad fuer geheime Anwenderschluessel.		*/
/*									*/
/*	Sonstiges:							*/
/*	  Dieses Protokoll erfordert das grosse Speichermodell.		*/
/*									*/
/************************************************************************/


PROCESS all;


DCL bits_partner, s, r integer;
DCL n, b integer;
DCL bits_user, t, anz, v, x, y integer;
DCL public, private charstring;
DCL user, partner charstring;
DCL auth boolean;
DCL q, g, h integer;
DCL w, u1, u2 integer;
DCL path charstring;
DCL p, e, d, wert integer;
DCL a, alpha, beta, k integer;
DCL message charstring;
DCL block, c, l integer;


START;

TASK user := 'Andreas';
TASK bits_user := 512;
TASK anz := 20;
TASK public  := 'f:/user/rieke/diplom/keys/public';
TASK private := 'f:/user/rieke/diplom/keys/private';

NEXTSTATE ready;


STATE ready;

  INPUT dif_hel ( bits_partner );
    TASK path := '/' // int2str ( bits_partner );
    TASK p := load_int ( public // path // '/dif_hel.p' );
    TASK s := load_int ( public // path // '/dif_hel.s' );
    
    TASK a := rand_max ( p );
    TASK alpha := mod_exp ( s, a, p );
    OUTPUT dif_hel_msg ( alpha );
    NEXTSTATE dif_hel;

  INPUT dsa ( partner, message, bits_partner, r, s );
    TASK path := '/' // int2str ( bits_partner );
    TASK q := load_int ( public // path // '/dsa.q' );
    TASK p := load_int ( public // path // '/dsa.p' );
    TASK g := load_int ( public // path // '/dsa.g' );
    TASK path := '/' // partner // path;
    TASK y := load_int ( public // '/' // path // '/dsa.y' );
    TASK h := hash_md5 ( message );
    TASK w := rcp ( s, q );
    TASK u1 := mod_mul ( h, w, q );
    TASK u2 := mod_mul ( r, w, q );
    TASK a := mod_exp ( g, u1, p );
    TASK b := mod_exp ( y, u2, p );
    TASK t := mod_mul ( a, b, p );
    TASK v := t mod q;
    DECISION v = r;
      (true):
        CALL writeln ( 'DSA-Uebertragung von ', partner, ' mit ', 
                       bits_partner, ' Bits: positiv.' );
      ELSE:
        CALL writeln ( 'DSA-Uebertragung von ', partner, ' mit ',
		       bits_partner, ' Bits: negativ.' );
    ENDDECISION;
    CALL writeln ( '  ', message );
    NEXTSTATE ready;

  INPUT el_gam_mul ( bits_partner );
    TASK path := '/' // int2str ( bits_partner );
    TASK p := load_int ( public  // path // '/el_gam.p' );
    TASK path := '/' // user // path;
    TASK b := load_int ( private // path // '/el_gam.b' );
    TASK message := '';
    NEXTSTATE el_gam_mul;

  INPUT el_gam_sin ( bits_partner, d, c );
    TASK path := '/' // int2str ( bits_partner );
    TASK p := load_int ( public  // path // '/el_gam.p' );
    TASK path := '/' // user // path;
    TASK b := load_int ( private // path // '/el_gam.b' );

    TASK k := mod_exp ( d, b, p );
    TASK l := rcp ( k, p );
    TASK block := mod_mul ( c, l, p );
    TASK message := unpack ( block );
    CALL writeln ( 'ElGamal-Einfachuebertragung mit ', bits_partner, 
		   ' Bits:' );
    CALL writeln ( '  ', message );
    NEXTSTATE ready;

  INPUT fia_sha_auth ( bits_partner );
    TASK n := load_int ( public  // '/' // int2str ( bits_partner ) //
			    '/fia_sha.n' );
    TASK s := load_int ( private // '/' // user // '/' // 
			    int2str ( bits_partner ) // '/fia_sha.s' );
    NEXTSTATE fia_sha_user_1;

  INPUT fia_sha_user ( partner );
    TASK auth := true;
    TASK t := anz;
    TASK n := load_int ( public // '/' // int2str ( bits_user ) //
			    '/fia_sha.n' );
    TASK v := load_int ( public // '/' // partner // '/' // 
			    int2str ( bits_user ) // '/fia_sha.v' );
    OUTPUT fia_sha_auth ( bits_user );
    OUTPUT fia_sha_next;
    NEXTSTATE fia_sha_auth_1;

  INPUT mas_omu_mul ( bits_partner );
    TASK message := '';
    TASK p := load_int ( public // '/' // int2str ( bits_partner ) //
			    '/mas_omu.prm' );
    TASK path := '/' // user // '/' // int2str ( bits_partner );
    TASK e := load_int ( private // path // '/mas_omu.e' );
    TASK d := load_int ( private // path // '/mas_omu.d' );
    NEXTSTATE mas_omu_mul;

  INPUT mas_omu_sin ( bits_partner, wert );
    TASK p := load_int ( public // '/' // int2str ( bits_partner ) //
			    '/mas_omu.prm' );
    TASK path := '/' // user // '/' // int2str ( bits_partner );
    TASK e := load_int ( private // path // '/mas_omu.e' );
    TASK d := load_int ( private // path // '/mas_omu.d' );
    TASK wert := mod_exp ( wert, e, p );
    OUTPUT mas_omu_sin_b ( wert );
    NEXTSTATE mas_omu_sin_2;

  INPUT plain ( bits_partner );
    TASK message := '';
    NEXTSTATE plain;

  INPUT quit;
    NEXTSTATE ready;

  INPUT rsa_mul ( bits_partner );
    TASK path := '/' // user // '/' // int2str ( bits_partner );
    TASK n := load_int ( public  // path // '/rsa.n' );
    TASK d := load_int ( private // path // '/rsa.d' );
    TASK message := '';
    NEXTSTATE rsa_mul;

  INPUT rsa_sin ( bits_partner, wert );
    TASK path := '/' // user // '/' // int2str ( bits_partner );
    TASK n := load_int ( public  // path // '/rsa.n' );
    TASK d := load_int ( private // path // '/rsa.d' );
    TASK wert := mod_exp ( wert, d, n );
    TASK message := unpack ( wert );
    CALL writeln ( 'RSA-Einfachuebertragung mit ', bits_partner, 
		   ' Bits:' );
    CALL writeln ( '  ', message );
    NEXTSTATE ready;

ENDSTATE;

STATE auth;
  INPUT quit;
    NEXTSTATE ready;
ENDSTATE;

STATE dif_hel;
  INPUT dif_hel_msg ( beta );
    TASK k := mod_exp ( beta, a, p );
    CALL writeln ( 'Diffie-Hellman-Schluesselaustausch mit ', 
		   bits_partner, ' Bits durchgefuehrt:' );
    CALL writeln ( '  ', k );
    NEXTSTATE ready;
ENDSTATE;
    
STATE el_gam_mul;
  INPUT el_gam_mul_next ( d, c );
    TASK k := mod_exp ( d, b, p );
    TASK l := rcp ( k, p );
    TASK block := mod_mul ( c, l, p );
    TASK message := message // unpack ( block );
    NEXTSTATE el_gam_mul;

  INPUT el_gam_mul_last;
    CALL writeln ( 'ElGamal-Mehrfachuebertragung mit ', bits_partner, 
		   ' Bits:' );
    CALL writeln ( '  ', message );
    NEXTSTATE ready;
ENDSTATE;

STATE fia_sha_auth_1;
  INPUT fia_sha_rand ( x );
    TASK b := rand_max ( 2 );
    OUTPUT fia_sha_bit ( b );
    NEXTSTATE fia_sha_auth_2;
ENDSTATE;

STATE fia_sha_auth_2;
  INPUT fia_sha_erg ( y );
    DECISION b = 0;
      (true):
        DECISION y * y mod n /= x;
          (true):
            TASK auth := false;
        ENDDECISION;
      ELSE:
        DECISION y * y mod n /= x * v mod n;
          (true):
	    TASK auth := false;
        ENDDECISION;
    ENDDECISION;
    TASK t := t - 1;
    DECISION t = 0;
      (true):
	OUTPUT fia_sha_last;
        DECISION auth;
          (true):
            CALL writeln ( 'Fiat-Shamir-Authentifikation von ', partner,
			   ' mit ', bits_user, ' Bits: positiv.' );
	    NEXTSTATE auth;
    	  ELSE:
	    CALL writeln ( 'Fiat-Shamir-Authentifikation von ', partner,
			   ' mit ', bits_user, ' Bits: negativ.' );
	    NEXTSTATE ready;
        ENDDECISION;  
      ELSE:
	OUTPUT fia_sha_next;
        NEXTSTATE fia_sha_auth_1;
    ENDDECISION;
ENDSTATE;

STATE fia_sha_user_1;
  INPUT fia_sha_next;
    TASK r := rand_max ( n );
    OUTPUT fia_sha_rand ( r * r mod n );
    NEXTSTATE fia_sha_user_2;
  
  INPUT fia_sha_last;
    OUTPUT quit;
    NEXTSTATE ready;
ENDSTATE;

STATE fia_sha_user_2;
  INPUT fia_sha_bit ( b );
    DECISION b = 0;
      (true): 
	OUTPUT fia_sha_erg ( r );
      ELSE:
	OUTPUT fia_sha_erg ( r * s mod n );
    ENDDECISION;
    NEXTSTATE fia_sha_user_1;
ENDSTATE;

STATE mas_omu_mul;
  INPUT mas_omu_mul_next ( wert );
    TASK wert := mod_exp ( wert, e, p );
    OUTPUT mas_omu_mul_a ( wert );
    NEXTSTATE mas_omu_mul_2;
  INPUT mas_omu_mul_last;
    CALL writeln ( 'Massey-Omura-Mehrfachuebertragung mit ', 
		   bits_partner, ' Bits:' );
    CALL writeln ( '  ', message );
    NEXTSTATE ready;
ENDSTATE;

STATE mas_omu_mul_2;
  INPUT mas_omu_mul_b ( wert );
    TASK wert := mod_exp ( wert, d, p );
    TASK message := message // unpack ( wert );
    NEXTSTATE mas_omu_mul;
ENDSTATE;

STATE mas_omu_sin_2;
  INPUT mas_omu_sin_c ( wert );
    TASK wert := mod_exp ( wert, d, p );
    TASK message := unpack ( wert );
    CALL writeln ( 'Massey-Omura-Einfachuebertragung mit ', 
		   bits_partner, ' Bits:' );
    CALL writeln ( '  ', message );
    NEXTSTATE ready;
ENDSTATE;

STATE plain;
  INPUT plain_next ( wert );
    TASK message := message // unpack ( wert );
    NEXTSTATE plain;
  
  INPUT plain_last;
    CALL writeln ( 'Klartextuebertragung mit ', bits_partner, ' Bits:' );
    CALL writeln ( '  ', message );
    NEXTSTATE ready;
ENDSTATE;

STATE rsa_mul;
  INPUT rsa_mul_next ( wert );
    TASK message := message // unpack ( mod_exp ( wert, d, n ) );
    NEXTSTATE rsa_mul;

  INPUT rsa_mul_last;
    CALL writeln ( 'RSA-Mehrfachuebertragung mit ', bits_partner, 
		   ' Bits:' );
    CALL writeln ( '  ', message );
    NEXTSTATE ready;
ENDSTATE;


ENDPROCESS;
