/************************************************************************/
/*									*/
/*	EL_GAM\RECEIVE.PRO				21.7.93		*/
/*									*/
/*	Andreas Rieke, Heusnerstr. 24, 42283 Wuppertal.			*/
/*									*/
/*	Protokoll zum Entschluesseln einer mit Hilfe des ElGamal -	*/
/*	Verschluesselungsschemas verschluesselten Nachricht oder	*/
/*	eines Blocks.							*/
/*									*/
/*	Vorgaben:							*/
/*	  user:    Name des Empfaengers.				*/
/*	  public:  Pfad fuer oeffentliche Schluessel.			*/
/*	  private: Pfad fuer geheime Anwenderschluessel.		*/
/*									*/
/************************************************************************/


PROCESS el_gam_receive;


DCL bits, block, c, d integer;
DCL user charstring;
DCL public, private, path charstring;
DCL p, b, k, l integer;
DCL message charstring;


START;

TASK user := 'Andreas';		/* Maximal 8 Zeichen.		*/
TASK public := 'f:/user/rieke/diplom/keys/public'; 
TASK private := 'f:/user/rieke/diplom/keys/private';

NEXTSTATE ready;


STATE ready;
  INPUT el_gam_mul ( bits );
    TASK path := '/' // int2str ( bits );
    TASK p := load_int ( public  // path // '/el_gam.p' );
    TASK path := '/' // user // path;
    TASK b := load_int ( private // path // '/el_gam.b' );

    TASK message := '';
    NEXTSTATE el_gam_mul;
  INPUT el_gam_sin ( bits, d, c );
    TASK path := '/' // int2str ( bits );
    TASK p := load_int ( public  // path // '/el_gam.p' );
    TASK path := '/' // user // path;
    TASK b := load_int ( private // path // '/el_gam.b' );

    TASK k := mod_exp ( d, b, p );
    TASK l := rcp ( k, p );
    TASK block := mod_mul ( c, l, p );
    TASK message := unpack ( block );
    CALL writeln ( 'ElGamal-Einfachuebertragung mit ', bits, ' Bits:' );
    CALL writeln ( '  ', message );
    NEXTSTATE ready;
ENDSTATE;

STATE el_gam_mul;
  INPUT el_gam_mul_next ( d, c );
    TASK k := mod_exp ( d, b, p );
    TASK l := rcp ( k, p );
    TASK block := mod_mul ( c, l, p );
    TASK message := message // unpack ( block );
    NEXTSTATE el_gam_mul;

  INPUT el_gam_mul_last;
    CALL writeln ( 'ElGamal - Mehrfachuebertragung mit ', bits, 
		   ' Bits:' );
    CALL writeln ( '  ', message );
    NEXTSTATE ready;
ENDSTATE;


ENDPROCESS;
