/************************************************************************/
/*									*/
/*	FIA_SHA\SIMPLE\PASSIVE.PRO			21.7.93		*/
/*									*/
/*	Andreas Rieke, Heusnerstr. 24, 42283 Wuppertal.			*/
/*									*/
/*	Dieses Protokoll beinhaltet alle passiven Fiat-Shamir-		*/
/*	Protokolle.							*/
/*	Der Partner kann also sowohl die Authentifizierung dieses	*/
/*	Anwenders alsauch seine eigene Authentifikation anregen.	*/
/*									*/
/*	Vorgaben:							*/
/*	  user:      Name dieses Anwenders.				*/
/*	  bits_user: Bei der Authentifikation des Partners werden 	*/
/*		     Schluessel dieser Laenge verwendet.		*/
/*	  anz:       Anzahl der Authentifikationsschritte bei der 	*/
/*		     Authentifikation des Partners.			*/
/*	  public:    Pfad fuer oeffentliche Schluessel.			*/
/*	  private:   Pfad fuer geheime Anwenderschluessel.		*/
/*									*/
/*	Sonstiges:							*/
/*	  Fuer dieses Protokoll muss das grosse Speichermodell benutzt	*/
/*	  werden.							*/
/*									*/
/************************************************************************/


PROCESS fia_sha_simple_passive;


DCL bits_partner, s, r integer;
DCL n, b integer;
DCL bits_user, t, anz, v, x, y integer;
DCL public, private charstring;
DCL user, partner charstring;
DCL auth boolean;


START;

TASK user := 'Andreas';
TASK bits_user := 512;
TASK anz := 20;
TASK public  := 'f:/user/rieke/diplom/keys/public';
TASK private := 'f:/user/rieke/diplom/keys/private';

NEXTSTATE ready;


STATE ready;
  INPUT fia_sha_auth ( bits_partner );
    TASK n := load_int ( public  // '/' // int2str ( bits_partner ) //
			    '/fia_sha.n' );
    TASK s := load_int ( private // '/' // user // '/' // 
			    int2str ( bits_partner ) // '/fia_sha.s' );
    NEXTSTATE fia_sha_user_1;
  INPUT fia_sha_user ( partner );
    TASK auth := true;
    TASK t := anz;
    TASK n := load_int ( public // '/' // int2str ( bits_user ) //
			    '/fia_sha.n' );
    TASK v := load_int ( public // '/' // partner // '/' // 
			    int2str ( bits_user ) // '/fia_sha.v' );
    OUTPUT fia_sha_auth ( bits_user );
    OUTPUT fia_sha_next;
    NEXTSTATE fia_sha_auth_1;

  INPUT quit;
    NEXTSTATE ready;
ENDSTATE;

STATE auth;
  INPUT quit;
    NEXTSTATE ready;
ENDSTATE;

STATE fia_sha_auth_1;
  INPUT fia_sha_rand ( x );
    TASK b := rand_max ( 2 );
    OUTPUT fia_sha_bit ( b );
    NEXTSTATE fia_sha_auth_2;
ENDSTATE;

STATE fia_sha_auth_2;
  INPUT fia_sha_erg ( y );
    DECISION b = 0;
      (true):
        DECISION y * y mod n /= x;
          (true):
            TASK auth := false;
        ENDDECISION;
      ELSE:
        DECISION y * y mod n /= x * v mod n;
          (true):
	    TASK auth := false;
        ENDDECISION;
    ENDDECISION;
    TASK t := t - 1;
    DECISION t = 0;
      (true):
	OUTPUT fia_sha_last;
        DECISION auth;
          (true):
            CALL writeln ( 'Fiat-Shamir-Authentifikation von ', partner,
			   ' mit ', bits_user, ' Bits: positiv.' );
	    NEXTSTATE auth;
    	  ELSE:
	    CALL writeln ( 'Fiat-Shamir-Authentifikation von ', partner,
			   ' mit ', bits_user, ' Bits: negativ.' );
	    NEXTSTATE ready;
        ENDDECISION;  
      ELSE:
	OUTPUT fia_sha_next;
        NEXTSTATE fia_sha_auth_1;
    ENDDECISION;
ENDSTATE;

STATE fia_sha_user_1;
  INPUT fia_sha_next;
    TASK r := rand_max ( n );
    OUTPUT fia_sha_rand ( r * r mod n );
    NEXTSTATE fia_sha_user_2;
  
  INPUT fia_sha_last;
    OUTPUT quit;
    NEXTSTATE ready;
ENDSTATE;

STATE fia_sha_user_2;
  INPUT fia_sha_bit ( b );
    DECISION b = 0;
      (true): 
	OUTPUT fia_sha_erg ( r );
      ELSE:
	OUTPUT fia_sha_erg ( r * s mod n );
    ENDDECISION;
    NEXTSTATE fia_sha_user_1;
ENDSTATE;


ENDPROCESS;
