/************************************************************************/
/*									*/
/*	MATH\SQRT\FAST_P.PRO				1. 2. 95	*/
/*									*/
/*	Andreas Rieke, Heusnerstr. 24, 42283 Wuppertal.			*/
/*									*/
/*	Protokoll zum modularen Quadratwurzelziehen.			*/
/*	( r * r ) mod p = a						*/
/*									*/
/*	Vorgaben:							*/
/*	  p: Modulus. p mu eine ungerade Primzahl sein.		*/
/*	  a: Quadrat. 0 < a < p.					*/
/*									*/
/*	Sonstiges:							*/
/*	  Der Algorithmus ist in 					*/
/*	    Koblitz, Neal: A Course in Number Theory and Cryptography.	*/
/*	    Springer, 1987.						*/
/*	  beschrieben.							*/
/*									*/
/*	  Falls p Primzahl ist und ( r * r ) mod p = a, so gilt auch	*/
/*	  ( ( p - r ) * ( p - r ) ) mod p = a.				*/
/*									*/
/************************************************************************/


PROCESS sqrt_fast_p;


DCL p, a integer;
DCL q, e, b integer;
DCL d, r, t, k, h integer;


START;

TASK p := 36013;
TASK a := 27412;

DECISION mod_exp ( a, ( p - 1 ) / 2, p ) = 1;
  (true):				/* Es gibt genau 2 Wurzeln.		*/
    TASK q := p - 1;
    TASK e := 0;
    MACRO repeat ( a );
      TASK q := q / 2;
      TASK e := e + 1;
    MACRO until ( q mod 2 = 1 );

    MACRO repeat ( b );
      TASK b := rand_max ( p );
    MACRO until ( mod_exp ( b, ( p - 1 ) / 2, p ) > 1 );
					/* b darf keine Wurzel mod p haben.	*/

    TASK d := mod_exp ( b, q, p );
    TASK r := mod_exp ( a, ( q + 1 ) / 2, p );
    TASK t := rcp ( a, p );
    TASK k := 0;

    DECISION e > 1;
      (true):
        MACRO repeat ( c );
          TASK h := r * r * t mod p;
          TASK h := mod_exp ( h, exp ( 2, e - k - 2 ), p );
          DECISION h = p - 1;
            (true):
              TASK r := r * mod_exp ( d, exp ( 2, k ), p ) mod p;
          ENDDECISION;
          TASK k := k + 1;
        MACRO until ( e - k <= 1 );
    ENDDECISION;

    DECISION r * r mod p = a;
      (true):
        CALL writeln ( r, ' * ', r, ' mod ', p, ' = ', a );
        DECISION ( p - r ) * ( p - r ) mod p = a;
          (true):
            CALL writeln ( p - r, ' * ', p - r, ' mod ', p, ' = ', a );
          ELSE:
            CALL writeln ( 'Fehler 1 !!!' );
        ENDDECISION;
      ELSE:
        CALL writeln ( 'Fehler 2 !!!' );
    ENDDECISION;
  ELSE:
    CALL writeln ( 'Fehler: ', a, ' hat keine Wurzel.' );
ENDDECISION;

STOP;

ENDPROCESS;
