/************************************************************************/
/*									*/
/*      SCHNORR\KEYS\USER.PRO                           22. 1. 95       */
/*									*/
/*	Andreas Rieke, Heusnerstr. 24, 42283 Wuppertal.			*/
/*									*/
/*	Berechnet die Anwenderschluessel fuer die Algorithmen von	*/
/*	Schnorr.							*/
/*									*/
/*	Vorgaben:							*/
/*	  user:    Name des Anwenders (Kurzform)			*/
/*	  id:	   Name des Anwenders (Langform)			*/
/*	  private: Pfad fuer geheime Schluessel.			*/
/*	  public:  Pfad fuer oeffentliche Schluessel.			*/
/*	  trust:   Pfad fuer Schluessel der Schluesselzentrale.		*/
/*									*/
/************************************************************************/


PROCESS schnorr_keys_user;


DCL user, id charstring;
DCL private, public, trust charstring;
DCL q, p, a, s integer;
DCL s_user, v_user integer;
DCL r, x, e, y integer;


START;

TASK user := 'Andreas';
TASK id := 'Andreas Rieke';

TASK private := 'f:/user/rieke/diplom/keys/private';
TASK public := 'f:/user/rieke/diplom/keys/public';
TASK trust := 'f:/user/rieke/diplom/keys/trust';

CALL mkdir ( private // '/' // user );
CALL mkdir ( public // '/' // user );

TASK q := load_int ( public // '/schnorr.q' );
TASK p := load_int ( public // '/schnorr.p' );
TASK a := load_int ( public // '/schnorr.a' );
TASK s := load_int ( trust // '/schnorr.s' );

TASK s_user := rand_max ( q ) + 1;
TASK v_user := rcp ( mod_exp ( a, s_user, p ), p );

TASK r := rand_max ( q ) + 1;
TASK x := mod_exp ( a, r, p );
TASK e := hash_md5 ( int2str ( x ) // int2str ( v_user ) // id );
TASK y := ( r + s * e ) mod q;

CALL store_str ( id, public // '/' // user // '/schnorr.i' );
CALL store ( s_user, private // '/' // user // '/schnorr.s' );
CALL store ( v_user, public // '/' // user // '/schnorr.v' );
CALL store ( e, public // '/' // user // '/schnorr.e' );
CALL store ( y, public // '/' // user // '/schnorr.y' );

STOP;


ENDPROCESS;
