/************************************************************************/
/*									*/
/*	USERKEYS.PRO					21.7.93		*/
/*									*/
/*	Andreas Rieke, Heusnerstr. 24, 42283 Wuppertal.			*/
/*									*/
/*	Berechnet alle Anwenderschluessel mit vorgegebener Laenge:	*/
/*	  EL_GAM\KEYS\USER.PRO						*/
/*	  FIA_SHA\KEYS\USER.PRO						*/
/*	  MAS_OMU\KEYS\USER.PRO						*/
/*	  RSA\KEYS.PRO							*/
/*									*/
/*	Vorgaben:							*/
/*	  user:    Name des Anwenders					*/
/*	  bits:    Laenge der Schluessel in binaerer Darstellung	*/
/*	  public:  Pfad fuer oeffentliche Schluessel			*/
/*	  private: Pfad fuer geheime Anwenderschluessel			*/
/*									*/
/************************************************************************/


PROCESS userkeys;


DCL bits integer;
DCL user, public, private, path charstring;
DCL q, p integer;
DCL s, b, beta integer;
DCL n, v integer;
DCL t, e, d integer;


START;

TASK bits := 512;			/* Anzahl der Bits.		*/
TASK user := 'Unknown';
TASK public := 'f:/user/rieke/diplom/keys/public';
TASK private := 'f:/user/rieke/diplom/keys/private';

/* Anwenderschluessel fuer das ElGamal-Verschluesselungsschema:		*/
TASK path := '/' // int2str ( bits );
TASK p := load_int ( public // path // '/el_gam.p' );
TASK s := load_int ( public // path // '/el_gam.s' );

TASK path := '/' // user // '/' // int2str ( bits );
CALL mkdir ( public  // path );
CALL mkdir ( private // path );

TASK b := rand_max ( p );
TASK beta := mod_exp ( s, b, p );

CALL store ( b, private // path // '/el_gam.b'    );
CALL store ( beta, public  // path // '/el_gam.bet' );

/* Anwenderschluessel fuer das Fiat-Shamir-Protokoll:			*/
TASK path := '/' // int2str ( bits );
TASK n := load_int ( public // path // '/fia_sha.n' );
TASK path := '/' // user // path;
CALL mkdir ( public  // path );
CALL mkdir ( private // path );

TASK s := rand_max ( n );		/* Zufallszahl berechnen.	*/
TASK v := mod_mul ( s, s, n );		/* und quadrieren.		*/

CALL store ( s, private // path // '/fia_sha.s' );
CALL store ( v, public  // path // '/fia_sha.v' );

/* Anwenderschluessel fuer das Massey-Omura-Schema:			*/
TASK path := '/' // user // '/' // int2str ( bits );
CALL mkdir ( private // path );		/* Fuer geheime Schluessel.	*/

TASK p := load_int ( public // '/' // int2str ( bits ) //
			'/mas_omu.prm' );

TASK t := p - 1;			/* Anzahl teilerfremde Zahlen.	*/

MACRO repeat ( a ); 
  TASK e := rand_max ( t );		/* Zufallszahl kleiner t.	*/
MACRO until ( gcd ( e, t ) = 1 );	/* e muss teilerfr. zu t sein.  */

TASK d := rcp ( e, t );			/* Modularer Kehrwert.		*/

CALL store ( e, private // path // '/mas_omu.e' );
CALL store ( d, private // path // '/mas_omu.d' );

/* Anwenderschluessel fuer das RSA-Verschluesselungsschema:		*/
TASK path := '/' // user // '/' // int2str ( bits );
CALL mkdir ( public  // path );		/* Fuer diesen Anwender.	*/
CALL mkdir ( private // path );

TASK p := rand_bits_prime ( bits / 2 );	
MACRO repeat ( a );
  TASK q := rand_bits_prime ( bits / 2 );
MACRO until ( p /= q );			/* p ungleich q.		*/

TASK n := p * q;	
TASK t := ( p - 1 ) * ( q - 1 );	/* Anzahl teilerfremde Zahlen.	*/

MACRO repeat ( b ); 
  TASK e := rand_max ( t );		/* Zufallszahl kleiner t.	*/
MACRO until ( gcd ( e, t ) = 1 );	/* e muss teilerfr. zu t sein.  */

TASK d := rcp ( e, t );			/* Modularer Kehrwert.		*/
	
CALL store ( n, public  // path // '/rsa.n' );
CALL store ( e, public  // path // '/rsa.e' );
CALL store ( d, private // path // '/rsa.d' );

STOP;


ENDPROCESS;
